from invoke import task
import boto3
from {{cookiecutter.project_name}}.core.utils import project_dir_name

@task(help={'bucket': "The name of the S3 bucket to use. If not provided, this is generated from the project directory name."})
def create(c, bucket=None):
    """
    Creates an S3 bucket to store data.
    """
    if bucket == None:
        bucket = "dvc-" + project_dir_name().replace("_","-")
    s3 = boto3.client("s3")
    s3.create_bucket(Bucket=bucket)

@task(pre=[create],help={'bucket': "The name of the S3 bucket to use for the DVC remote. If not provided, this is generated from the project directory name."})
def dvc(c, bucket=None):
    """
    Adds a DVC S3 remote as the default remote and creates the S3 bucket.
    """
    if bucket == None:
        bucket = "dvc-" + project_dir_name().replace("_","-")
    c.run("dvc remote add -d s3 s3://{}/".format(bucket))



from invoke import task
import sys
import subprocess

@task(help={"relative_path": "Relative path to the notebook"})
def run(c, relative_path):
    """
    Run a notebook from the command line.
    """
    c.run("jupyter nbconvert --clear-output --ExecutePreprocessor.timeout=1000 --execute {}".format(relative_path))
    sys.stdout.flush()



from invoke import Collection, task

# Default tasks are in core.tasks
import {{cookiecutter.project_name}}.core.tasks.app as app
import {{cookiecutter.project_name}}.core.tasks.notebooks as notebooks
import {{cookiecutter.project_name}}.core.tasks.setup as setup
import {{cookiecutter.project_name}}.core.tasks.destroy as destroy



# Add default task collections
ns = Collection()
ns.add_collection(app)
ns = Collection(app)
ns.add_collection(notebooks)
ns.add_collection(setup)
ns.add_collection(destroy)



from invoke import task
import whisk.git as git

@task
def start(c):
    """
    Start the web service serving model inference.
    """
    c.run("honcho -f app/Procfile.dev start", pty=True)

@task(help={'name': "Name of the Heroku app."})
def create(c,name):
    """
    Create a Heroku app for the web service.
    """
    if git.has_unstaged_changes():
        print("This project has uncommitted changes.\nPlease add and commit the files to the Git repo, then retry:\n\ngit add .\ngit commit -m 'First Commit'")
        exit(1)
    c.run("heroku create -a {}".format(name))
    c.run("heroku buildpacks:add -a {} https://github.com/heroku/heroku-buildpack-multi-procfile".format(name))
    c.run("heroku buildpacks:add heroku/python")
    c.run("heroku config:set PROCFILE=app/Procfile")
    c.run("git push heroku master")

@task
def destroy(c):
    """
    Delete the Heroku app.
    """
    c.run("heroku apps:destroy", pty=True)

@task
def test(c):
    """
    Run the web service unit tests.
    """
    c.run("python app/test.py")



"""Main module."""
import whisk
from cookiecutter.main import cookiecutter
import os
from os.path import dirname, realpath
# https://docs.python.org/3/library/pathlib.html
# Object-oriented filesystem paths
from pathlib import Path

def root_module_dir():
    """
    Returns a Path object with the root whisk module directory.
    """
    filepath = realpath(__file__)
    return Path(filepath).parents[0]

def cookiecutter_template_dir():
    return str(root_module_dir() / 'template/')

def create(project_name, output_dir=".", setup=None, force=False):
    """
    Creates a whisk project.
    """
    extra_content = {
        "project_name": project_name,
        "whisk_dependency": "whisk>={}".format(whisk.__version__)
    }
    if setup is not None:
        extra_content["setup"] = setup
    cookiecutter(cookiecutter_template_dir(),
                no_input=True,
                overwrite_if_exists=force,
                output_dir=output_dir,
                extra_context=extra_content)



from invoke import Collection, task
import {{cookiecutter.project_name}}.core.tasks

# Core Tasks
ns = Collection.from_module({{cookiecutter.project_name}}.core.tasks)

### Add project-specific tasks below. ###
# See http://docs.pyinvoke.org/en/stable/getting-started.html

from {{cookiecutter.project_name}}.models.model import Model

@task(help={'data': "A single record to perform model inference"})
def predict(c, data):
    """
    Invokes the model.
    """
    model = Model()
    print(model.predict([data]))

model = Collection('model')
model.add_task(predict)
ns.add_collection(model)



from invoke import task
import boto3
from {{cookiecutter.project_name}}.core.utils import project_dir_name

@task(help={'bucket': "The name of the S3 bucket to destroy. If not provided, this is generated from the project directory name."})
def delete(c, bucket=None):
    """
    Delete the S3 bucket used for data storage.
    """
    if bucket == None:
        bucket = "dvc-" + project_dir_name().replace("_","-")
    bucket_resource = boto3.resource('s3').Bucket(bucket)
    bucket_resource.objects.all().delete()
    s3 = boto3.client("s3")
    s3.delete_bucket(Bucket=bucket)

@task(help={'bucket': "The name of the S3 bucket to use for the DVC remote. If not provided, this is generated from the project directory name."})
def dvc(c, bucket=None):
    """
    Removes the DVC S3 remote.
    """
    if bucket == None:
        bucket = "dvc-" + project_dir_name().replace("_","-")
    c.run("dvc remote remove s3".format(bucket))



import click

@click.group()
def app():
    pass

@app.command()
def start():
    """Start the HTTP web service."""
    print("starting")

@app.command()
def create(name):
    """
    Create a Heroku app for the web service.
    """



"""
Console script for whisk.
Groups structure is based on https://stackoverflow.com/a/39416589/1234395.
"""
import sys
import click
from whisk import whisk
from whisk.project import Project
from .commands.app import app
from .commands.setup import setup

def load_project_commands(click_group):
    """
    If inside a whisk project directory, project-specific commands are added to the cli.
    """
    if Project().in_project():
        click_group.add_command(app)
        click_group.add_command(setup)

@click.group()
def main():
    pass

@main.command()
@click.argument('name')
@click.option('--setup/--no-setup', default=True, help="Run setup script after creating directory structure.")
@click.option('--force/--no-force', default=False, help="Overwrite the contents of output directory if it exists.")
@click.option('-o', '--output_dir', default=".", help="The parent directory that will contain the project.")
def create(name, setup, output_dir, force):
    """Creates a Whisk project in the NAME directory."""
    whisk.create(name, setup=setup, output_dir=output_dir, force=force)

load_project_commands(main)

if __name__ == "__main__":
    main()



from setuptools import find_packages, setup

def list_reqs(fname='requirements.txt'):
    """
    Adds the packages listed in `fname` as package requirements.

    By default this is the project's main requirements.txt file. To provide
    a smaller set of packages required only for model predictions, add a
    `src/requirements.txt` file w/the subset of required packages and load this file instead.
    """
    with open(fname) as fd:
        return fd.read().splitlines()

setup(
    name='{{cookiecutter.project_name}}',
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    version='0.1.0',
    include_package_data=True,
    # TODO - show how to include dependencies with install_requires
    # install_requires=list_reqs(),
    entry_points={
        'console_scripts': [
            '{{cookiecutter.project_name}}={{cookiecutter.project_name}}.cli.main:cli',
        ],
    },
    description='A short description of the project.',
    author='Your name (or your organization/company/team)',
    python_requires='>=3.6',
)



import os
# https://docs.python.org/3/library/pathlib.html
# Object-oriented filesystem paths
from pathlib import Path

class Project:
    """
    Abstracts project-level attributes into a class.
    """

    @classmethod
    def from_module(cls,path):
        path_obj = Path(path)
        obj = cls(path_obj.parents[2],module_dir=path_obj.parents[0])
        return obj

    def __init__(self, dir=os.getcwd(), module_dir=None):
        """
        Initializes a project from the existing whisk project directory `dir`.
        """
        self.dir = dir
        self.path = Path(self.dir)
        self.module_dir = module_dir
        if not self.module_dir:
            self.module_dir = self.path / "src/{}".format(self.slug())
        self.artifacts_dir = self.module_dir / "artifacts"
        if self.in_project():
            self.data_dir = self.path/ "data"

    def validate_in_project(self):
        if not self.in_project():
            raise OSError("{} is not a whisk project directory.".format(self.dir))

    def in_project(self):
        """
        Returns True if the current working directory is root whisk
        project directory.
        """
        return (self.path / ".whisk").is_dir()

    def slug(self):
        """
        Returns the project slug derived from the project directory name.
        """
        return self.path.stem
